<?php
namespace Pinpoint\Feefo\Model;
/**
 * Class FeefoServiceReview
 * @package Pinpoint\Feefo\Model
 */
class FeefoServiceReview extends \Magento\Framework\Model\AbstractModel implements \Pinpoint\Feefo\Api\Data\FeefoServiceReviewInterface, \Magento\Framework\DataObject\IdentityInterface
{
    /**
     * @var string
     */
    const CACHE_TAG = 'pinpoint_feefo_feefoservicereview';

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    protected $reviewCollection;

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    protected $reviews;

    /**
     * construct
     */
    protected function _construct()
    {
        $this->_init('Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview');
    }

    /**
     * @return array
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    /**
     * @param bool $forceUpdate
     * @return mixed|null|\Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    public function getServiceReviews($forceUpdate = false)
    {
        if ($this->reviews && !$forceUpdate) {
            return $this->reviews;
        }

        $this->reviewCollection = $reviews = $this->getCollection();
        if ($reviews->count()) {
            $this->reviews = $reviews;
            return $reviews;
        }

        return null;
    }

    /**
     * @return mixed|null|\Pinpoint\Feefo\Model\ResourceModel\FeefoServiceReview\Collection
     */
    public function getSliderReviews()
    {
        return $this->getCollection()
            ->setPageSize(5)
            ->setCurPage(1)
            ->setOrder('created_at', 'DESC');
    }

    /**
     * @return int
     */
    public function getReviewCount()
    {
        if (!$this->reviewCollection) {
            $this->getServiceReviews(true);
        }

        return $this->reviewCollection->count();
    }

    /**
     * @return float|int
     */
    public function getAverageRating()
    {
        if (!$this->reviewCollection) {
            $this->getServiceReviews();
        }

        $ratings = [
            '0' => 0,
            '1' => 0,
            '2' => 0,
            '3' => 0,
            '4' => 0,
            '5' => 0
        ];

        foreach ($this->reviewCollection as $review) {
            $ratings[$review->getData('rating')] += 1;
        }

        if (!array_sum(array_values($ratings))) {
            return 0;
        }

        $rating = (5*$ratings['5'] + 4*$ratings['4'] + 3*$ratings['3'] + 2*$ratings['2'] + 1*$ratings['1']) / array_sum(array_values($ratings));
        return floor($rating * 2) / 2;
    }
}
